@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: shell.h,v 1.27 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: shell.h,v 1.28 2009-07-03 19:36:57 qbix79 Exp $ */
 
 #ifndef DOSBOX_SHELL_H
 #define DOSBOX_SHELL_H
@@ -44,7 +44,7 @@ class DOS_Shell;
 
 class BatchFile {
 public:
-	BatchFile(DOS_Shell * host,char const* const name, char const * const cmd_line);
+	BatchFile(DOS_Shell * host,char const* const resolved_name,char const* const entered_name, char const * const cmd_line);
 	virtual ~BatchFile();
 	virtual bool ReadLine(char * line);
 	bool Goto(char * where);
@@ -55,6 +55,7 @@ public:
 	DOS_Shell * shell;
 	BatchFile * prev;
 	CommandLine * cmd;
+	std::string filename;
 };
 
 class AutoexecEditor;
