@@ -2104,7 +2104,9 @@ restart_prefix:
 			case 0xbf:dyn_mov_ev_gw(true);break;
 
 			default:
-				DYN_LOG("Unhandled dual opcode 0F%02X",dual_code);
+#if DYN_LOG
+				LOG_MSG("Unhandled dual opcode 0F%02X",dual_code);
+#endif
 				goto illegalopcode;
 			}
 		}break;
@@ -2670,7 +2672,9 @@ restart_prefix:
 			}}
 			break;
 		default:
-//			DYN_LOG("Dynamic unhandled opcode %X",opcode);
+#if DYN_LOG
+//			LOG_MSG("Dynamic unhandled opcode %X",opcode);
+#endif
 			goto illegalopcode;
 		}
 	}
