@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: risc_armv4le-thumb-iw.h,v 1.3 2009/05/16 21:52:47 c2woody Exp $ */
+/* $Id: risc_armv4le-thumb-iw.h,v 1.5 2009-06-27 12:51:10 c2woody Exp $ */
 
 
 /* ARMv4 (little endian) backend by M-HT (thumb version with data pool, requires -mthumb-interwork switch when compiling dosbox) */
@@ -25,15 +25,10 @@
 // temporary "lo" registers
 #define templo1 HOST_v3
 #define templo2 HOST_v4
-
-// temporary "lo" register - value must be preserved when using it
-#define templosav HOST_a3
-
-// temporary "hi" register
-#define temphi1 HOST_ip
+#define templo3 HOST_v2
 
 // register that holds function return values
-#define FC_RETOP HOST_v2
+#define FC_RETOP HOST_a1
 
 // register used for address calculations,
 #define FC_ADDR HOST_v1			// has to be saved across calls, see DRC_PROTECT_ADDR_REG
@@ -44,6 +39,9 @@
 // register that holds the second parameter
 #define FC_OP2 HOST_a2
 
+// special register that holds the third parameter for _R3 calls (byte accessible)
+#define FC_OP3 HOST_a4
+
 // register that holds byte-accessible temporary values
 #define FC_TMP_BA1 HOST_a1
 
@@ -83,8 +81,6 @@
 #define ADD_IMM8(dst, imm) (0x3000 + ((dst) << 8) + (imm) )
 // add dst, src1, src2
 #define ADD_REG(dst, src1, src2) (0x1800 + (dst) + ((src1) << 3) + ((src2) << 6) )
-// add dst, src
-#define ADD_LO_HI(dst, src) (0x4440 + (dst) + (((src) - HOST_r8) << 3) )
 // add dst, pc, #imm		@	0 <= imm < 1024	&	imm mod 4 = 0
 #define ADD_LO_PC_IMM(dst, imm) (0xa000 + ((dst) << 8) + ((imm) >> 2) )
 // sub dst, src1, src2
@@ -499,11 +495,9 @@ static void gen_extend_word(bool sign,HostReg reg) {
 
 // add a 32bit value from memory to a full register
 static void gen_add(HostReg reg,void* op) {
+	gen_mov_word_to_reg(templo3, op, 1);
 	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, reg) );      // mov temphi1, reg
-	gen_mov_word_to_reg(reg, op, 1);
-	cache_checkinstr(2);
-	cache_addw( ADD_LO_HI(reg, temphi1) );      // add reg, temphi1
+	cache_addw( ADD_REG(reg, reg, templo3) );      // add reg, reg, templo3
 }
 
 // add a 32bit constant value to a full register
@@ -525,12 +519,8 @@ static void gen_and_imm(HostReg reg,Bit32u imm) {
 
 // move a 32bit constant value into memory
 static void gen_mov_direct_dword(void* dest,Bit32u imm) {
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
-	gen_mov_dword_to_reg_imm(templosav, imm);
-	gen_mov_word_from_reg(templosav, dest, 1);
-	cache_checkinstr(2);
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
+	gen_mov_dword_to_reg_imm(templo3, imm);
+	gen_mov_word_from_reg(templo3, dest, 1);
 }
 
 // move an address into memory
@@ -541,19 +531,15 @@ static void INLINE gen_mov_direct_ptr(void* dest,DRC_PTR_SIZE_IM imm) {
 // add an 8bit constant value to a dword memory value
 static void gen_add_direct_byte(void* dest,Bit8s imm) {
 	if(!imm) return;
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
 	gen_mov_dword_to_reg_imm(templo2, (Bit32u)dest);
-	gen_mov_word_to_reg_helper(templosav, dest, 1, templo2);
+	gen_mov_word_to_reg_helper(templo3, dest, 1, templo2);
 	cache_checkinstr(2);
 	if (imm >= 0) {
-		cache_addw( ADD_IMM8(templosav, (Bit32s)imm) );      // add templosav, #(imm)
+		cache_addw( ADD_IMM8(templo3, (Bit32s)imm) );      // add templo3, #(imm)
 	} else {
-		cache_addw( SUB_IMM8(templosav, -((Bit32s)imm)) );      // sub templosav, #(-imm)
+		cache_addw( SUB_IMM8(templo3, -((Bit32s)imm)) );      // sub templo3, #(-imm)
 	}
-	gen_mov_word_from_reg_helper(templosav, dest, 1, templo2);
-	cache_checkinstr(2);
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
+	gen_mov_word_from_reg_helper(templo3, dest, 1, templo2);
 }
 
 // add a 32bit (dword==true) or 16bit (dword==false) constant value to a memory value
@@ -563,38 +549,30 @@ static void gen_add_direct_word(void* dest,Bit32u imm,bool dword) {
 		gen_add_direct_byte(dest,(Bit8s)imm);
 		return;
 	}
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
 	gen_mov_dword_to_reg_imm(templo2, (Bit32u)dest);
-	gen_mov_word_to_reg_helper(templosav, dest, dword, templo2);
+	gen_mov_word_to_reg_helper(templo3, dest, dword, templo2);
 	if (dword) {
 		gen_mov_dword_to_reg_imm(templo1, imm);
 	} else {
 		gen_mov_word_to_reg_imm(templo1, (Bit16u)imm);
 	}
 	cache_checkinstr(2);
-	cache_addw( ADD_REG(templosav, templosav, templo1) );      // add templosav, templosav, templo1
-	gen_mov_word_from_reg_helper(templosav, dest, dword, templo2);
-	cache_checkinstr(2);
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
+	cache_addw( ADD_REG(templo3, templo3, templo1) );      // add templo3, templo3, templo1
+	gen_mov_word_from_reg_helper(templo3, dest, dword, templo2);
 }
 
 // subtract an 8bit constant value from a dword memory value
 static void gen_sub_direct_byte(void* dest,Bit8s imm) {
 	if(!imm) return;
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
 	gen_mov_dword_to_reg_imm(templo2, (Bit32u)dest);
-	gen_mov_word_to_reg_helper(templosav, dest, 1, templo2);
+	gen_mov_word_to_reg_helper(templo3, dest, 1, templo2);
 	cache_checkinstr(2);
 	if (imm >= 0) {
-		cache_addw( SUB_IMM8(templosav, (Bit32s)imm) );      // sub templosav, #(imm)
+		cache_addw( SUB_IMM8(templo3, (Bit32s)imm) );      // sub templo3, #(imm)
 	} else {
-		cache_addw( ADD_IMM8(templosav, -((Bit32s)imm)) );      // add templosav, #(-imm)
+		cache_addw( ADD_IMM8(templo3, -((Bit32s)imm)) );      // add templo3, #(-imm)
 	}
-	gen_mov_word_from_reg_helper(templosav, dest, 1, templo2);
-	cache_checkinstr(2);
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
+	gen_mov_word_from_reg_helper(templo3, dest, 1, templo2);
 }
 
 // subtract a 32bit (dword==true) or 16bit (dword==false) constant value from a memory value
@@ -604,20 +582,16 @@ static void gen_sub_direct_word(void* dest,Bit32u imm,bool dword) {
 		gen_sub_direct_byte(dest,(Bit8s)imm);
 		return;
 	}
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
 	gen_mov_dword_to_reg_imm(templo2, (Bit32u)dest);
-	gen_mov_word_to_reg_helper(templosav, dest, dword, templo2);
+	gen_mov_word_to_reg_helper(templo3, dest, dword, templo2);
 	if (dword) {
 		gen_mov_dword_to_reg_imm(templo1, imm);
 	} else {
 		gen_mov_word_to_reg_imm(templo1, (Bit16u)imm);
 	}
 	cache_checkinstr(2);
-	cache_addw( SUB_REG(templosav, templosav, templo1) );      // sub templosav, templosav, templo1
-	gen_mov_word_from_reg_helper(templosav, dest, dword, templo2);
-	cache_checkinstr(2);
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
+	cache_addw( SUB_REG(templo3, templo3, templo1) );      // sub templo3, templo3, templo1
+	gen_mov_word_from_reg_helper(templo3, dest, dword, templo2);
 }
 
 // effective address calculation, destination is dest_reg
@@ -670,12 +644,11 @@ static void gen_call_function_helper(void * func) {
 	// after_call:
 
 	// thumb state from now on
-	cache_addw( MOV_REG(FC_RETOP, HOST_a1) );      // mov FC_RETOP, a1
 }
 
 // generate a call to a parameterless function
 static void INLINE gen_call_function_raw(void * func) {
-	cache_checkinstr(14);
+	cache_checkinstr(12);
 	gen_call_function_helper(func);
 }
 
@@ -683,13 +656,13 @@ static void INLINE gen_call_function_raw(void * func) {
 // note: the parameters are loaded in the architecture specific way
 // using the gen_load_param_ functions below
 static Bit32u INLINE gen_call_function_setup(void * func,Bitu paramcount,bool fastcall=false) {
-	cache_checkinstr(14);
+	cache_checkinstr(12);
 	Bit32u proc_addr = (Bit32u)cache.pos;
 	gen_call_function_helper(func);
 	return proc_addr;
 	// if proc_addr is on word  boundary ((proc_addr & 0x03) == 0)
-	//   then length of generated code is 14 bytes
-	//   otherwise length of generated code is 12 bytes
+	//   then length of generated code is 12 bytes
+	//   otherwise length of generated code is 10 bytes
 }
 
 #if (1)
@@ -720,33 +693,31 @@ static void INLINE gen_load_param_mem(Bitu mem,Bitu param) {
 
 // jump to an address pointed at by ptr, offset is in imm
 static void gen_jmp_ptr(void * ptr,Bits imm=0) {
-	cache_checkinstr(2);
-	cache_addw( MOV_HI_LO(temphi1, templosav) );      // mov temphi1, templosav
-	gen_mov_word_to_reg(templosav, ptr, 1);
+	gen_mov_word_to_reg(templo3, ptr, 1);
 
 	if (imm) {
 		gen_mov_dword_to_reg_imm(templo2, imm);
 		cache_checkinstr(2);
-		cache_addw( ADD_REG(templosav, templosav, templo2) );      // add templosav, templosav, templo2
+		cache_addw( ADD_REG(templo3, templo3, templo2) );      // add templo3, templo3, templo2
 	}
 
 #if (1)
 // (*ptr) should be word aligned 
 	if ((imm & 0x03) == 0) {
-		cache_checkinstr(8);
-		cache_addw( LDR_IMM(templo2, templosav, 0) );      // ldr templo2, [templosav]
+		cache_checkinstr(6);
+		cache_addw( LDR_IMM(templo2, templo3, 0) );      // ldr templo2, [templo3]
 	} else
 #endif
 	{
-		cache_checkinstr(26);
-		cache_addw( LDRB_IMM(templo2, templosav, 0) );      // ldrb templo2, [templosav]
-		cache_addw( LDRB_IMM(templo1, templosav, 1) );      // ldrb templo1, [templosav, #1]
+		cache_checkinstr(24);
+		cache_addw( LDRB_IMM(templo2, templo3, 0) );      // ldrb templo2, [templo3]
+		cache_addw( LDRB_IMM(templo1, templo3, 1) );      // ldrb templo1, [templo3, #1]
 		cache_addw( LSL_IMM(templo1, templo1, 8) );      // lsl templo1, templo1, #8
 		cache_addw( ORR(templo2, templo1) );      // orr templo2, templo1
-		cache_addw( LDRB_IMM(templo1, templosav, 2) );      // ldrb templo1, [templosav, #2]
+		cache_addw( LDRB_IMM(templo1, templo3, 2) );      // ldrb templo1, [templo3, #2]
 		cache_addw( LSL_IMM(templo1, templo1, 16) );      // lsl templo1, templo1, #16
 		cache_addw( ORR(templo2, templo1) );      // orr templo2, templo1
-		cache_addw( LDRB_IMM(templo1, templosav, 3) );      // ldrb templo1, [templosav, #3]
+		cache_addw( LDRB_IMM(templo1, templo3, 3) );      // ldrb templo1, [templo3, #3]
 		cache_addw( LSL_IMM(templo1, templo1, 24) );      // lsl templo1, templo1, #24
 		cache_addw( ORR(templo2, templo1) );      // orr templo2, templo1
 	}
@@ -754,8 +725,6 @@ static void gen_jmp_ptr(void * ptr,Bits imm=0) {
 	// increase jmp address to keep thumb state
 	cache_addw( ADD_IMM3(templo2, templo2, 1) );      // add templo2, templo2, #1
 
-	cache_addw( MOV_LO_HI(templosav, temphi1) );      // mov templosav, temphi1
-
 	cache_addw( BX(templo2) );      // bx templo2
 }
 
@@ -895,8 +864,7 @@ static void gen_run_code(void) {
 
 // return from a function
 static void gen_return_function(void) {
-	cache_checkinstr(6);
-	cache_addw( MOV_REG(HOST_a1, FC_RETOP) );      // mov a1, FC_RETOP
+	cache_checkinstr(4);
 	cache_addw(0xbc08);      // pop {r3}
 	cache_addw( BX(HOST_r3) );      // bx r3
 }
@@ -930,35 +898,32 @@ static void gen_fill_function_ptr(Bit8u * pos,void* fct_ptr,Bitu flags_type) {
 			case t_ADDb:
 			case t_ADDw:
 			case t_ADDd:
-				*(Bit16u*)pos=ADD_REG(FC_RETOP, HOST_a1, HOST_a2);	// add FC_RETOP, a1, a2
-				*(Bit16u*)(pos+2)=B_FWD(8);							// b after_call (pc+8)
+				*(Bit16u*)pos=ADD_REG(HOST_a1, HOST_a1, HOST_a2);	// add a1, a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_ORb:
 			case t_ORw:
 			case t_ORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ORR(FC_RETOP, HOST_a2);			// orr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=ORR(HOST_a1, HOST_a2);				// orr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_ANDb:
 			case t_ANDw:
 			case t_ANDd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=AND(FC_RETOP, HOST_a2);			// and FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=AND(HOST_a1, HOST_a2);				// and a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_SUBb:
 			case t_SUBw:
 			case t_SUBd:
-				*(Bit16u*)pos=SUB_REG(FC_RETOP, HOST_a1, HOST_a2);	// sub FC_RETOP, a1, a2
-				*(Bit16u*)(pos+2)=B_FWD(8);							// b after_call (pc+8)
+				*(Bit16u*)pos=SUB_REG(HOST_a1, HOST_a1, HOST_a2);	// sub a1, a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_XORb:
 			case t_XORw:
 			case t_XORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=EOR(FC_RETOP, HOST_a2);			// eor FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=EOR(HOST_a1, HOST_a2);				// eor a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_CMPb:
 			case t_CMPw:
@@ -966,117 +931,107 @@ static void gen_fill_function_ptr(Bit8u * pos,void* fct_ptr,Bitu flags_type) {
 			case t_TESTb:
 			case t_TESTw:
 			case t_TESTd:
-				*(Bit16u*)pos=B_FWD(10);							// b after_call (pc+10)
+				*(Bit16u*)pos=B_FWD(8);								// b after_call (pc+8)
 				break;
 			case t_INCb:
 			case t_INCw:
 			case t_INCd:
-				*(Bit16u*)pos=ADD_IMM3(FC_RETOP, HOST_a1, 1);		// add FC_RETOP, a1, #1
-				*(Bit16u*)(pos+2)=B_FWD(8);							// b after_call (pc+8)
+				*(Bit16u*)pos=ADD_IMM3(HOST_a1, HOST_a1, 1);		// add a1, a1, #1
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_DECb:
 			case t_DECw:
 			case t_DECd:
-				*(Bit16u*)pos=SUB_IMM3(FC_RETOP, HOST_a1, 1);		// sub FC_RETOP, a1, #1
-				*(Bit16u*)(pos+2)=B_FWD(8);							// b after_call (pc+8)
+				*(Bit16u*)pos=SUB_IMM3(HOST_a1, HOST_a1, 1);		// sub a1, a1, #1
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_SHLb:
 			case t_SHLw:
 			case t_SHLd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=LSL_REG(FC_RETOP, HOST_a2);		// lsl FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=LSL_REG(HOST_a1, HOST_a2);			// lsl a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_SHRb:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 24);		// lsl FC_RETOP, a1, #24
-				*(Bit16u*)(pos+2)=LSR_IMM(FC_RETOP, FC_RETOP, 24);	// lsr FC_RETOP, FC_RETOP, #24
-				*(Bit16u*)(pos+4)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+6)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=LSR_IMM(HOST_a1, HOST_a1, 24);	// lsr a1, a1, #24
+				*(Bit16u*)(pos+6)=NOP;								// nop
+				*(Bit16u*)(pos+8)=LSR_REG(HOST_a1, HOST_a2);		// lsr a1, a2
+				*(Bit16u*)(pos+10)=NOP;								// nop
 				break;
 			case t_SHRw:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 16);		// lsl FC_RETOP, a1, #16
-				*(Bit16u*)(pos+2)=LSR_IMM(FC_RETOP, FC_RETOP, 16);	// lsr FC_RETOP, FC_RETOP, #16
-				*(Bit16u*)(pos+4)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+6)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=LSR_IMM(HOST_a1, HOST_a1, 16);	// lsr a1, a1, #16
+				*(Bit16u*)(pos+6)=NOP;								// nop
+				*(Bit16u*)(pos+8)=LSR_REG(HOST_a1, HOST_a2);		// lsr a1, a2
+				*(Bit16u*)(pos+10)=NOP;								// nop
 				break;
 			case t_SHRd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=LSR_REG(HOST_a1, HOST_a2);			// lsr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_SARb:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 24);		// lsl FC_RETOP, a1, #24
-				*(Bit16u*)(pos+2)=ASR_IMM(FC_RETOP, FC_RETOP, 24);	// asr FC_RETOP, FC_RETOP, #24
-				*(Bit16u*)(pos+4)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+6)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=ASR_IMM(HOST_a1, HOST_a1, 24);	// asr a1, a1, #24
+				*(Bit16u*)(pos+6)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ASR_REG(HOST_a1, HOST_a2);		// asr a1, a2
+				*(Bit16u*)(pos+10)=NOP;								// nop
 				break;
 			case t_SARw:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 16);		// lsl FC_RETOP, a1, #16
-				*(Bit16u*)(pos+2)=ASR_IMM(FC_RETOP, FC_RETOP, 16);	// asr FC_RETOP, FC_RETOP, #16
-				*(Bit16u*)(pos+4)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+6)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=ASR_IMM(HOST_a1, HOST_a1, 16);	// asr a1, a1, #16
+				*(Bit16u*)(pos+6)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ASR_REG(HOST_a1, HOST_a2);		// asr a1, a2
+				*(Bit16u*)(pos+10)=NOP;								// nop
 				break;
 			case t_SARd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=ASR_REG(HOST_a1, HOST_a2);			// asr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			case t_RORb:
 				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
-				*(Bit16u*)(pos+2)=LSR_IMM(FC_RETOP, HOST_a1, 8);	// lsr FC_RETOP, a1, #8
-				*(Bit16u*)(pos+4)=ORR(HOST_a1, FC_RETOP);			// orr a1, FC_RETOP
-				*(Bit16u*)(pos+6)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+8)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+10)=NOP;								// nop
-				*(Bit16u*)(pos+12)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
+				*(Bit16u*)(pos+2)=LSR_IMM(templo1, HOST_a1, 8);		// lsr templo1, a1, #8
+				*(Bit16u*)(pos+4)=ORR(HOST_a1, templo1);			// orr a1, templo1
+				*(Bit16u*)(pos+6)=LSR_IMM(templo1, HOST_a1, 16);	// lsr templo1, a1, #16
+				*(Bit16u*)(pos+8)=ORR(HOST_a1, templo1);			// orr a1, templo1
+				*(Bit16u*)(pos+10)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
 				break;
 			case t_RORw:
 				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
-				*(Bit16u*)(pos+2)=NOP;								// nop
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+10)=NOP;								// nop
-				*(Bit16u*)(pos+12)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
+				*(Bit16u*)(pos+2)=LSR_IMM(templo1, HOST_a1, 16);	// lsr templo1, a1, #16
+				*(Bit16u*)(pos+4)=NOP;								// nop
+				*(Bit16u*)(pos+6)=ORR(HOST_a1, templo1);			// orr a1, templo1
+				*(Bit16u*)(pos+8)=NOP;								// nop
+				*(Bit16u*)(pos+10)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
 				break;
 			case t_RORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=ROR_REG(HOST_a1, HOST_a2);			// ror a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
-			/*case t_ROLb:
-				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
-				*(Bit16u*)(pos+2)=NEG(templo1, HOST_a2);			// neg templo1, a2
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, HOST_a1, 8);	// lsr FC_RETOP, a1, #8
-				*(Bit16u*)(pos+6)=ADD_IMM8(templo1, 32);			// add templo1, #32
-				*(Bit16u*)(pos+8)=ORR(HOST_a1, FC_RETOP);			// orr a1, FC_RETOP
-				*(Bit16u*)(pos+10)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+12)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+14)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
-				break;*/
 			case t_ROLw:
 				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
-				*(Bit16u*)(pos+2)=NEG(templo1, HOST_a2);			// neg templo1, a2
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+6)=ADD_IMM8(templo1, 32);			// add templo1, #32
-				*(Bit16u*)(pos+8)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+10)=NOP;								// nop
-				*(Bit16u*)(pos+12)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
+				*(Bit16u*)(pos+2)=NEG(HOST_a2, HOST_a2);			// neg a2, a2
+				*(Bit16u*)(pos+4)=LSR_IMM(templo1, HOST_a1, 16);	// lsr templo1, a1, #16
+				*(Bit16u*)(pos+6)=ADD_IMM8(HOST_a2, 32);			// add a2, #32
+				*(Bit16u*)(pos+8)=ORR(HOST_a1, templo1);			// orr a1, templo1
+				*(Bit16u*)(pos+10)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
 				break;
 			case t_ROLd:
-				*(Bit16u*)pos=NEG(templo1, HOST_a2);				// neg templo1, a2
-				*(Bit16u*)(pos+2)=MOV_REG(FC_RETOP, HOST_a1);		// mov FC_RETOP, a1
-				*(Bit16u*)(pos+4)=NOP;								// nop
-				*(Bit16u*)(pos+6)=ADD_IMM8(templo1, 32);			// add templo1, #32
-				*(Bit16u*)(pos+8)=NOP;								// nop
-				*(Bit16u*)(pos+10)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
-				*(Bit16u*)(pos+12)=NOP;								// nop
+				*(Bit16u*)pos=NEG(HOST_a2, HOST_a2);				// neg a2, a2
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=ADD_IMM8(HOST_a2, 32);			// add a2, #32
+				*(Bit16u*)(pos+6)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
+				*(Bit16u*)(pos+10)=NOP;								// nop
 				break;
 			case t_NEGb:
 			case t_NEGw:
 			case t_NEGd:
-				*(Bit16u*)pos=NEG(FC_RETOP, HOST_a1);				// neg FC_RETOP, a1
-				*(Bit16u*)(pos+2)=B_FWD(8);							// b after_call (pc+8)
+				*(Bit16u*)pos=NEG(HOST_a1, HOST_a1);				// neg a1, a1
+				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
 				break;
 			default:
 				*(Bit32u*)( ( ((Bit32u) (*pos)) << 2 ) + ((Bit32u)pos + 4) ) = (Bit32u)fct_ptr;		// simple_func
@@ -1090,35 +1045,32 @@ static void gen_fill_function_ptr(Bit8u * pos,void* fct_ptr,Bitu flags_type) {
 			case t_ADDb:
 			case t_ADDw:
 			case t_ADDd:
-				*(Bit16u*)pos=ADD_REG(FC_RETOP, HOST_a1, HOST_a2);	// add FC_RETOP, a1, a2
-				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=ADD_REG(HOST_a1, HOST_a1, HOST_a2);	// add a1, a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_ORb:
 			case t_ORw:
 			case t_ORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ORR(FC_RETOP, HOST_a2);			// orr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=ORR(HOST_a1, HOST_a2);				// orr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_ANDb:
 			case t_ANDw:
 			case t_ANDd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=AND(FC_RETOP, HOST_a2);			// and FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=AND(HOST_a1, HOST_a2);				// and a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_SUBb:
 			case t_SUBw:
 			case t_SUBd:
-				*(Bit16u*)pos=SUB_REG(FC_RETOP, HOST_a1, HOST_a2);	// sub FC_RETOP, a1, a2
-				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=SUB_REG(HOST_a1, HOST_a1, HOST_a2);	// sub a1, a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_XORb:
 			case t_XORw:
 			case t_XORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=EOR(FC_RETOP, HOST_a2);			// eor FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=EOR(HOST_a1, HOST_a2);				// eor a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_CMPb:
 			case t_CMPw:
@@ -1126,121 +1078,85 @@ static void gen_fill_function_ptr(Bit8u * pos,void* fct_ptr,Bitu flags_type) {
 			case t_TESTb:
 			case t_TESTw:
 			case t_TESTd:
-				*(Bit16u*)pos=B_FWD(8);								// b after_call (pc+8)
+				*(Bit16u*)pos=B_FWD(6);								// b after_call (pc+6)
 				break;
 			case t_INCb:
 			case t_INCw:
 			case t_INCd:
-				*(Bit16u*)pos=ADD_IMM3(FC_RETOP, HOST_a1, 1);		// add FC_RETOP, a1, #1
-				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=ADD_IMM3(HOST_a1, HOST_a1, 1);		// add a1, a1, #1
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_DECb:
 			case t_DECw:
 			case t_DECd:
-				*(Bit16u*)pos=SUB_IMM3(FC_RETOP, HOST_a1, 1);		// sub FC_RETOP, a1, #1
-				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=SUB_IMM3(HOST_a1, HOST_a1, 1);		// sub a1, a1, #1
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_SHLb:
 			case t_SHLw:
 			case t_SHLd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=LSL_REG(FC_RETOP, HOST_a2);		// lsl FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSL_REG(HOST_a1, HOST_a2);			// lsl a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_SHRb:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 24);		// lsl FC_RETOP, a1, #24
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
 				*(Bit16u*)(pos+2)=NOP;								// nop
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, FC_RETOP, 24);	// lsr FC_RETOP, FC_RETOP, #24
+				*(Bit16u*)(pos+4)=LSR_IMM(HOST_a1, HOST_a1, 24);	// lsr a1, a1, #24
 				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+10)=NOP;								// nop
+				*(Bit16u*)(pos+8)=LSR_REG(HOST_a1, HOST_a2);		// lsr a1, a2
 				break;
 			case t_SHRw:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 16);		// lsl FC_RETOP, a1, #16
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
 				*(Bit16u*)(pos+2)=NOP;								// nop
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, FC_RETOP, 16);	// lsr FC_RETOP, FC_RETOP, #16
+				*(Bit16u*)(pos+4)=LSR_IMM(HOST_a1, HOST_a1, 16);	// lsr a1, a1, #16
 				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+10)=NOP;								// nop
+				*(Bit16u*)(pos+8)=LSR_REG(HOST_a1, HOST_a2);		// lsr a1, a2
 				break;
 			case t_SHRd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=LSR_REG(FC_RETOP, HOST_a2);		// lsr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
+				*(Bit16u*)pos=LSR_REG(HOST_a1, HOST_a2);			// lsr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_SARb:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 24);		// lsl FC_RETOP, a1, #24
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
 				*(Bit16u*)(pos+2)=NOP;								// nop
-				*(Bit16u*)(pos+4)=ASR_IMM(FC_RETOP, FC_RETOP, 24);	// asr FC_RETOP, FC_RETOP, #24
+				*(Bit16u*)(pos+4)=ASR_IMM(HOST_a1, HOST_a1, 24);	// asr a1, a1, #24
 				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+10)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ASR_REG(HOST_a1, HOST_a2);		// asr a1, a2
 				break;
 			case t_SARw:
-				*(Bit16u*)pos=LSL_IMM(FC_RETOP, HOST_a1, 16);		// lsl FC_RETOP, a1, #16
+				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
 				*(Bit16u*)(pos+2)=NOP;								// nop
-				*(Bit16u*)(pos+4)=ASR_IMM(FC_RETOP, FC_RETOP, 16);	// asr FC_RETOP, FC_RETOP, #16
+				*(Bit16u*)(pos+4)=ASR_IMM(HOST_a1, HOST_a1, 16);	// asr a1, a1, #16
 				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+10)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ASR_REG(HOST_a1, HOST_a2);		// asr a1, a2
 				break;
 			case t_SARd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ASR_REG(FC_RETOP, HOST_a2);		// asr FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
-				break;
-			case t_RORb:
-				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
-				*(Bit16u*)(pos+2)=LSR_IMM(FC_RETOP, HOST_a1, 8);	// lsr FC_RETOP, a1, #8
-				*(Bit16u*)(pos+4)=ORR(HOST_a1, FC_RETOP);			// orr a1, FC_RETOP
-				*(Bit16u*)(pos+6)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+8)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+10)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
+				*(Bit16u*)pos=ASR_REG(HOST_a1, HOST_a2);			// asr a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_RORw:
 				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
-				*(Bit16u*)(pos+2)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
+				*(Bit16u*)(pos+2)=LSR_IMM(templo1, HOST_a1, 16);	// lsr templo1, a1, #16
 				*(Bit16u*)(pos+4)=NOP;								// nop
-				*(Bit16u*)(pos+6)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+8)=NOP;								// nop
-				*(Bit16u*)(pos+10)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
+				*(Bit16u*)(pos+6)=ORR(HOST_a1, templo1);			// orr a1, templo1
+				*(Bit16u*)(pos+8)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
 				break;
 			case t_RORd:
-				*(Bit16u*)pos=MOV_REG(FC_RETOP, HOST_a1);			// mov FC_RETOP, a1
-				*(Bit16u*)(pos+2)=ROR_REG(FC_RETOP, HOST_a2);		// ror FC_RETOP, a2
-				*(Bit16u*)(pos+4)=B_FWD(4);							// b after_call (pc+4)
-				break;
-			/*case t_ROLb:
-				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 24);		// lsl a1, a1, #24
-				*(Bit16u*)(pos+2)=NEG(templo1, HOST_a2);			// neg templo1, a2
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, HOST_a1, 8);	// lsr FC_RETOP, a1, #8
-				*(Bit16u*)(pos+6)=ADD_IMM8(templo1, 32);			// add templo1, #32
-				*(Bit16u*)(pos+8)=ORR(HOST_a1, FC_RETOP);			// orr a1, FC_RETOP
-				*(Bit16u*)(pos+10)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+12)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+14)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
-				break;*/
-			case t_ROLw:
-				*(Bit16u*)pos=LSL_IMM(HOST_a1, HOST_a1, 16);		// lsl a1, a1, #16
-				*(Bit16u*)(pos+2)=NEG(templo1, HOST_a2);			// neg templo1, a2
-				*(Bit16u*)(pos+4)=LSR_IMM(FC_RETOP, HOST_a1, 16);	// lsr FC_RETOP, a1, #16
-				*(Bit16u*)(pos+6)=ADD_IMM8(templo1, 32);			// add templo1, #32
-				*(Bit16u*)(pos+8)=ORR(FC_RETOP, HOST_a1);			// orr FC_RETOP, a1
-				*(Bit16u*)(pos+10)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
+				*(Bit16u*)pos=ROR_REG(HOST_a1, HOST_a2);			// ror a1, a2
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			case t_ROLd:
-				*(Bit16u*)pos=NEG(templo1, HOST_a2);				// neg templo1, a2
-				*(Bit16u*)(pos+2)=MOV_REG(FC_RETOP, HOST_a1);		// mov FC_RETOP, a1
-				*(Bit16u*)(pos+4)=ADD_IMM8(templo1, 32);			// add templo1, #32
+				*(Bit16u*)pos=NEG(HOST_a2, HOST_a2);				// neg a2, a2
+				*(Bit16u*)(pos+2)=NOP;								// nop
+				*(Bit16u*)(pos+4)=ADD_IMM8(HOST_a2, 32);			// add a2, #32
 				*(Bit16u*)(pos+6)=NOP;								// nop
-				*(Bit16u*)(pos+8)=ROR_REG(FC_RETOP, templo1);		// ror FC_RETOP, templo1
-				*(Bit16u*)(pos+10)=NOP;								// nop
+				*(Bit16u*)(pos+8)=ROR_REG(HOST_a1, HOST_a2);		// ror a1, a2
 				break;
 			case t_NEGb:
 			case t_NEGw:
 			case t_NEGd:
-				*(Bit16u*)pos=NEG(FC_RETOP, HOST_a1);				// neg FC_RETOP, a1
-				*(Bit16u*)(pos+2)=B_FWD(6);							// b after_call (pc+6)
+				*(Bit16u*)pos=NEG(HOST_a1, HOST_a1);				// neg a1, a1
+				*(Bit16u*)(pos+2)=B_FWD(4);							// b after_call (pc+4)
 				break;
 			default:
 				*(Bit32u*)( ( ((Bit32u) (*pos)) << 2 ) + ((Bit32u)pos + 2) ) = (Bit32u)fct_ptr;		// simple_func
