@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -170,7 +170,6 @@
 	CASE_D(0x63)												/* ARPL Ed,Rd */
 		{
 			if (((cpu.pmode) && (reg_flags & FLAG_VM)) || (!cpu.pmode)) goto illegal_opcode;
-			FillFlags();
 			GetRMrw;
 			if (rm >= 0xc0 ) {
 				GetEArd;Bitu new_sel=(Bit16u)*eard;
@@ -398,12 +397,10 @@
 			continue;
 		}
 	CASE_D(0x9c)												/* PUSHFD */
-		FillFlags();
 		if (CPU_PUSHF(true)) RUNEXCEPTION();
 		break;
 	CASE_D(0x9d)												/* POPFD */
 		if (CPU_POPF(true)) RUNEXCEPTION();
-		lflags.type=t_UNKNOWN;
 #if CPU_TRAP_CHECK
 		if (GETFLAG(TF)) {	
 			cpudecoder=CPU_Core_Normal_Trap_Run;
@@ -465,14 +462,18 @@
 		continue;
 	CASE_D(0xc4)												/* LES */
 		{	
-			GetRMrd;GetEAa;
+			GetRMrd;
+			if (rm >= 0xc0) goto illegal_opcode;
+			GetEAa;
 			if (CPU_SetSegGeneral(es,LoadMw(eaa+4))) RUNEXCEPTION();
 			*rmrd=LoadMd(eaa);
 			break;
 		}
 	CASE_D(0xc5)												/* LDS */
 		{	
-			GetRMrd;GetEAa;
+			GetRMrd;
+			if (rm >= 0xc0) goto illegal_opcode;
+			GetEAa;
 			if (CPU_SetSegGeneral(ds,LoadMw(eaa+4))) RUNEXCEPTION();
 			*rmrd=LoadMd(eaa);
 			break;
@@ -492,7 +493,7 @@
 		}
 		break;
 	CASE_D(0xc9)												/* LEAVE */
-		reg_esp&=~cpu.stack.mask;
+		reg_esp&=cpu.stack.notmask;
 		reg_esp|=(reg_ebp&cpu.stack.mask);
 		reg_ebp=Pop_32();
 		break;
@@ -511,7 +512,6 @@
 		}
 	CASE_D(0xcf)												/* IRET */
 		{
-			FillFlags();
 			CPU_IRET(true,GETIP);
 #if CPU_TRAP_CHECK
 			if (GETFLAG(TF)) {	
@@ -709,8 +709,8 @@
 				else {GetEAa;Push_32(LoadMd(eaa));}
 				break;
 			default:
-				E_Exit("CPU:66:GRP5:Illegal call %2X",which);
-				break;
+				LOG(LOG_CPU,LOG_ERROR)("CPU:66:GRP5:Illegal call %2X",which);
+				goto illegal_opcode;
 			}
 			break;
 		}
