@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -687,14 +687,18 @@
 		continue;
 	CASE_W(0xc4)												/* LES */
 		{	
-			GetRMrw;GetEAa;
+			GetRMrw;
+			if (rm >= 0xc0) goto illegal_opcode;
+			GetEAa;
 			if (CPU_SetSegGeneral(es,LoadMw(eaa+2))) RUNEXCEPTION();
 			*rmrw=LoadMw(eaa);
 			break;
 		}
 	CASE_W(0xc5)												/* LDS */
 		{	
-			GetRMrw;GetEAa;
+			GetRMrw;
+			if (rm >= 0xc0) goto illegal_opcode;
+			GetEAa;
 			if (CPU_SetSegGeneral(ds,LoadMw(eaa+2))) RUNEXCEPTION();
 			*rmrw=LoadMw(eaa);
 			break;
@@ -1161,8 +1165,8 @@
 				else {GetEAa;Push_16(LoadMw(eaa));}
 				break;
 			default:
-				E_Exit("CPU:GRP5:Illegal Call %2X",which);
-				break;
+				LOG(LOG_CPU,LOG_ERROR)("CPU:GRP5:Illegal Call %2X",which);
+				goto illegal_opcode;
 			}
 			break;
 		}
