@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: fpu_instructions_x86.h,v 1.7 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: fpu_instructions_x86.h,v 1.7 2009-05-27 09:15:41 qbix79 Exp $ */
 
 
 // #define WEAK_EXCEPTIONS
@@ -661,9 +661,9 @@
 			"fstpt		(%2, %%eax)		\n"	\
 			"movw		%0, %%ax		\n"	\
 			"sahf						\n"	\
-			"jp			argument_too_large1		\n"	\
+			"jp			1f				\n"	\
 			"fstpt		(%2, %1)		\n"	\
-			"argument_too_large1:		"	\
+			"1:							"	\
 			:	"=m" (new_sw)				\
 			:	"r" (TOP), "r" (fpu.p_regs)	\
 			:	"eax", "cc", "memory"		\
@@ -687,9 +687,9 @@
 			"fstpt		(%2, %%eax)		\n"	\
 			"movw		%0, %%ax		\n"	\
 			"sahf						\n"	\
-			"jp			argument_too_large2		\n"	\
+			"jp			1f				\n"	\
 			"fstpt		(%2, %1)		\n"	\
-			"argument_too_large2:		"	\
+			"1:							"	\
 			:	"=m" (new_sw)				\
 			:	"r" (TOP), "r" (fpu.p_regs)	\
 			:	"eax", "cc", "memory"		\
@@ -1162,12 +1162,12 @@ static void FPU_FLD_F80(PhysPt addr) {
 
 static void FPU_FLD_I16(PhysPt addr,Bitu store_to) {
 	fpu.p_regs[8].m1 = (Bit32u)mem_readw(addr);
-	FPUD_LOAD(fild,WORD,)
+	FPUD_LOAD(fild,WORD,s)
 }
 
 static void FPU_FLD_I16_EA(PhysPt addr) {
 	fpu.p_regs[8].m1 = (Bit32u)mem_readw(addr);
-	FPUD_LOAD_EA(fild,WORD,)
+	FPUD_LOAD_EA(fild,WORD,s)
 }
 
 static void FPU_FLD_I32(PhysPt addr,Bitu store_to) {
@@ -1212,7 +1212,7 @@ static void FPU_FST_F80(PhysPt addr) {
 }
 
 static void FPU_FST_I16(PhysPt addr) {
-	FPUD_STORE(fistp,WORD,)
+	FPUD_STORE(fistp,WORD,s)
 	mem_writew(addr,(Bit16u)fpu.p_regs[8].m1);
 }
 
