@@ -411,7 +411,7 @@ static void MIXER_Mix_NoSound(void) {
 	mixer.done=0;
 }
 
-static void MIXER_CallBack(void * userdata, Uint8 *stream, int len) {
+static void SDLCALL MIXER_CallBack(void * userdata, Uint8 *stream, int len) {
 	Bitu need=(Bitu)len/MIXER_SSIZE;
 	Bit16s * output=(Bit16s *)stream;
 	Bitu reduce;
@@ -455,7 +455,7 @@ static void MIXER_CallBack(void * userdata, Uint8 *stream, int len) {
 			if(diff > (mixer.min_needed<<1)) diff = mixer.min_needed<<1;
 			if(diff > (mixer.min_needed>>1))
 				mixer.tick_add = ((mixer.freq-(diff/5)) << MIXER_SHIFT)/1000;
-			else if (diff > (mixer.min_needed>>4))
+			else if (diff > (mixer.min_needed>>2))
 				mixer.tick_add = ((mixer.freq-(diff>>3)) << MIXER_SHIFT)/1000;
 			else
 				mixer.tick_add = (mixer.freq<< MIXER_SHIFT)/1000;
