@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: sblaster.cpp,v 1.73 2009/04/25 07:02:28 qbix79 Exp $ */
+/* $Id: sblaster.cpp,v 1.78 2009-10-25 16:22:22 c2woody Exp $ */
 
 #include <iomanip>
 #include <sstream>
@@ -62,8 +62,8 @@ bool MIDI_Available(void);
 #define SB_SH	14
 #define SB_SH_MASK	((1 << SB_SH)-1)
 
-enum {DSP_S_RESET,DSP_S_NORMAL,DSP_S_HIGHSPEED};
-enum SB_TYPES {SBT_NONE=0,SBT_1=1,SBT_PRO1=2,SBT_2=3,SBT_PRO2=4,SBT_16=6};
+enum {DSP_S_RESET,DSP_S_RESET_WAIT,DSP_S_NORMAL,DSP_S_HIGHSPEED};
+enum SB_TYPES {SBT_NONE=0,SBT_1=1,SBT_PRO1=2,SBT_2=3,SBT_PRO2=4,SBT_16=6,SBT_GB=7};
 enum SB_IRQS {SB_IRQ_8,SB_IRQ_16,SB_IRQ_MPU};
 
 enum DSP_MODES {
@@ -117,6 +117,7 @@ struct SB_INFO {
 		Bit8u cmd_in_pos;
 		Bit8u cmd_in[DSP_BUFSIZE];
 		struct {
+			Bit8u lastval;
 			Bit8u data[DSP_BUFSIZE];
 			Bitu pos,used;
 		} in,out;
@@ -145,7 +146,7 @@ struct SB_INFO {
 	struct {
 		Bitu base;
 		Bitu irq;
-		Bitu dma8,dma16;
+		Bit8u dma8,dma16;
 	} hw;
 	struct {
 		Bits value;
@@ -246,13 +247,22 @@ static void DSP_SetSpeaker(bool how) {
 
 static INLINE void SB_RaiseIRQ(SB_IRQS type) {
 	LOG(LOG_SB,LOG_NORMAL)("Raising IRQ");
-	PIC_ActivateIRQ(sb.hw.irq);
 	switch (type) {
 	case SB_IRQ_8:
+		if (sb.irq.pending_8bit) {
+//			LOG_MSG("SB: 8bit irq pending");
+			return;
+		}
 		sb.irq.pending_8bit=true;
+		PIC_ActivateIRQ(sb.hw.irq);
 		break;
 	case SB_IRQ_16:
+		if (sb.irq.pending_16bit) {
+//			LOG_MSG("SB: 16bit irq pending");
+			return;
+		}
 		sb.irq.pending_16bit=true;
+		PIC_ActivateIRQ(sb.hw.irq);
 		break;
 	default:
 		break;
@@ -316,7 +326,7 @@ static INLINE Bit8u decode_ADPCM_4_sample(Bit8u sample,Bit8u & reference,Bits& s
 	Bits ref = reference + scaleMap[samp];
 	if (ref > 0xff) reference = 0xff;
 	else if (ref < 0x00) reference = 0x00;
-	else reference = ref;
+	else reference = (Bit8u)(ref&0xff);
 	scale = (scale + adjustMap[samp]) & 0xff;
 	
 	return reference;
@@ -345,7 +355,7 @@ static INLINE Bit8u decode_ADPCM_2_sample(Bit8u sample,Bit8u & reference,Bits& s
 	Bits ref = reference + scaleMap[samp];
 	if (ref > 0xff) reference = 0xff;
 	else if (ref < 0x00) reference = 0x00;
-	else reference = ref;
+	else reference = (Bit8u)(ref&0xff);
 	scale = (scale + adjustMap[samp]) & 0xff;
 
 	return reference;
@@ -377,7 +387,7 @@ INLINE Bit8u decode_ADPCM_3_sample(Bit8u sample,Bit8u & reference,Bits& scale) {
 	Bits ref = reference + scaleMap[samp];
 	if (ref > 0xff) reference = 0xff;
 	else if (ref < 0x00) reference = 0x00;
-	else reference = ref;
+	else reference = (Bit8u)(ref&0xff);
 	scale = (scale + adjustMap[samp]) & 0xff;
 
 	return reference;
@@ -573,7 +583,7 @@ static void DSP_ChangeMode(DSP_MODES mode) {
 	sb.mode=mode;
 }
 
-static void DSP_RaiseIRQEvent(Bitu val) {
+static void DSP_RaiseIRQEvent(Bitu /*val*/) {
 	SB_RaiseIRQ(SB_IRQ_8);
 }
 
@@ -678,13 +688,23 @@ static void DSP_AddData(Bit8u val) {
 }
 
 
+static void DSP_FinishReset(Bitu /*val*/) {
+	DSP_FlushData();
+	DSP_AddData(0xaa);
+	sb.dsp.state=DSP_S_NORMAL;
+}
+
 static void DSP_Reset(void) {
 	LOG(LOG_SB,LOG_ERROR)("DSP:Reset");
 	PIC_DeActivateIRQ(sb.hw.irq);
+
 	DSP_ChangeMode(MODE_NONE);
+	DSP_FlushData();
 	sb.dsp.cmd_len=0;
 	sb.dsp.in.pos=0;
 	sb.dsp.write_busy=0;
+	PIC_RemoveEvents(DSP_FinishReset);
+
 	sb.dma.left=0;
 	sb.dma.total=0;
 	sb.dma.stereo=false;
@@ -693,6 +713,7 @@ static void DSP_Reset(void) {
 	sb.dma.mode=DSP_DMA_NONE;
 	sb.dma.remain_size=0;
 	if (sb.dma.chan) sb.dma.chan->Clear_Request();
+
 	sb.freq=22050;
 	sb.time_constant=45;
 	sb.dac.used=0;
@@ -706,29 +727,28 @@ static void DSP_Reset(void) {
 	PIC_RemoveEvents(END_DMA_Event);
 }
 
-
 static void DSP_DoReset(Bit8u val) {
-	if ((val&1)!=0) {
+	if (((val&1)!=0) && (sb.dsp.state!=DSP_S_RESET)) {
 //TODO Get out of highspeed mode
 		DSP_Reset();
 		sb.dsp.state=DSP_S_RESET;
-	} else {
-		DSP_FlushData();
-		DSP_AddData(0xaa);
-		sb.dsp.state=DSP_S_NORMAL;
+	} else if (((val&1)==0) && (sb.dsp.state==DSP_S_RESET)) {	// reset off
+		sb.dsp.state=DSP_S_RESET_WAIT;
+		PIC_RemoveEvents(DSP_FinishReset);
+		PIC_AddEvent(DSP_FinishReset,20.0f/1000.0f,0);	// 20 microseconds
 	}
 }
 
-static void DSP_E2_DMA_CallBack(DmaChannel * chan, DMAEvent event) {
+static void DSP_E2_DMA_CallBack(DmaChannel * /*chan*/, DMAEvent event) {
 	if (event==DMA_UNMASKED) {
-		Bit8u val=sb.e2.value;
+		Bit8u val=(Bit8u)(sb.e2.value&0xff);
 		DmaChannel * chan=GetDMAChannel(sb.hw.dma8);
 		chan->Register_Callback(0);
 		chan->Write(1,&val);
 	}
 }
 
-static void DSP_ADC_CallBack(DmaChannel * chan, DMAEvent event) {
+static void DSP_ADC_CallBack(DmaChannel * /*chan*/, DMAEvent event) {
 	if (event!=DMA_UNMASKED) return;
 	Bit8u val=128;
 	DmaChannel * ch=GetDMAChannel(sb.hw.dma8);
@@ -918,7 +938,7 @@ static void DSP_DoCommand(void) {
 		DSP_FlushData();
 		switch (sb.type) {
 		case SBT_1:
-			DSP_AddData(0x1);DSP_AddData(0x1);break;
+			DSP_AddData(0x1);DSP_AddData(0x05);break;
 		case SBT_2:
 			DSP_AddData(0x2);DSP_AddData(0x1);break;
 		case SBT_PRO1:
@@ -975,6 +995,8 @@ static void DSP_DoCommand(void) {
 		LOG(LOG_SB,LOG_ERROR)("DSP:Unimplemented auto-init DMA ADPCM command %2X",sb.dsp.cmd);
 		break;
 	case 0x20:
+		DSP_AddData(0x7f);   // fake silent input for Creative parrot
+		break;
 	case 0x2c:
 	case 0x98: case 0x99: /* Documented only for DSP 2.x and 3.x */
 	case 0xa0: case 0xa8: /* Documented only for DSP 3.x */
@@ -1047,14 +1069,13 @@ static void DSP_DoWrite(Bit8u val) {
 
 static Bit8u DSP_ReadData(void) {
 /* Static so it repeats the last value on succesive reads (JANGLE DEMO) */
-	static Bit8u data = 0;
 	if (sb.dsp.out.used) {
-		data=sb.dsp.out.data[sb.dsp.out.pos];
+		sb.dsp.out.lastval=sb.dsp.out.data[sb.dsp.out.pos];
 		sb.dsp.out.pos++;
 		if (sb.dsp.out.pos>=DSP_BUFSIZE) sb.dsp.out.pos-=DSP_BUFSIZE;
 		sb.dsp.out.used--;
 	}
-	return data;
+	return sb.dsp.out.lastval;
 }
 
 //The soundblaster manual says 2.0 Db steps but we'll go for a bit less
@@ -1337,7 +1358,7 @@ static Bit8u CTMIXER_Read(void) {
 }
 
 
-static Bitu read_sb(Bitu port,Bitu iolen) {
+static Bitu read_sb(Bitu port,Bitu /*iolen*/) {
 	switch (port-sb.hw.base) {
 	case MIXER_INDEX:
 		return sb.mixer.index;
@@ -1347,7 +1368,10 @@ static Bitu read_sb(Bitu port,Bitu iolen) {
 		return DSP_ReadData();
 	case DSP_READ_STATUS:
 		//TODO See for high speed dma :)
-		sb.irq.pending_8bit=false;
+		if (sb.irq.pending_8bit)  {
+			sb.irq.pending_8bit=false;
+			PIC_DeActivateIRQ(sb.hw.irq);
+		}
 		if (sb.dsp.out.used) return 0xff;
 		else return 0x7f;
 	case DSP_ACK_16BIT:
@@ -1360,6 +1384,7 @@ static Bitu read_sb(Bitu port,Bitu iolen) {
 			if (sb.dsp.write_busy & 8) return 0xff;
 			return 0x7f;
 		case DSP_S_RESET:
+		case DSP_S_RESET_WAIT:
 			return 0xff;
 		}
 		return 0xff;
@@ -1372,19 +1397,20 @@ static Bitu read_sb(Bitu port,Bitu iolen) {
 	return 0xff;
 }
 
-static void write_sb(Bitu port,Bitu val,Bitu iolen) {
+static void write_sb(Bitu port,Bitu val,Bitu /*iolen*/) {
+	Bit8u val8=(Bit8u)(val&0xff);
 	switch (port-sb.hw.base) {
 	case DSP_RESET:
-		DSP_DoReset(val);
+		DSP_DoReset(val8);
 		break;
 	case DSP_WRITE_DATA:
-		DSP_DoWrite(val);
+		DSP_DoWrite(val8);
 		break;
 	case MIXER_INDEX:
-		sb.mixer.index=val;
+		sb.mixer.index=val8;
 		break;
 	case MIXER_DATA:
-		CTMIXER_Write(val);
+		CTMIXER_Write(val8);
 		break;
 	default:
 		LOG(LOG_SB,LOG_NORMAL)("Unhandled write to SB Port %4X",port);
@@ -1392,8 +1418,21 @@ static void write_sb(Bitu port,Bitu val,Bitu iolen) {
 	}
 }
 
-static void adlib_gusforward(Bitu port,Bitu val,Bitu iolen) {
-	adlib_commandreg=val;
+static void adlib_gusforward(Bitu /*port*/,Bitu val,Bitu /*iolen*/) {
+	adlib_commandreg=(Bit8u)(val&0xff);
+}
+
+bool SB_Get_Address(Bitu& sbaddr, Bitu& sbirq, Bitu& sbdma) {
+	sbaddr=0;
+	sbirq =0;
+	sbdma =0;
+	if (sb.type == SBT_NONE) return false;
+	else {
+		sbaddr=sb.hw.base;
+		sbirq =sb.hw.irq;
+		sbdma = sb.hw.dma8;
+		return true;
+	}
 }
 
 static void SBLASTER_CallBack(Bitu len) {
@@ -1422,6 +1461,7 @@ static void SBLASTER_CallBack(Bitu len) {
 		break;
 	}
 }
+
 class SBLASTER: public Module_base {
 private:
 	/* Data */
@@ -1429,6 +1469,7 @@ private:
 	IO_WriteHandleObject WriteHandler[0x10];
 	AutoexecObject autoexecline;
 	MixerObject MixerChan;
+	OPL_Mode oplmode;
 
 	/* Support Functions */
 	void Find_Type_And_Opl(Section_prop* config,SB_TYPES& type, OPL_Mode& opl_mode){
@@ -1438,6 +1479,7 @@ private:
 		else if (!strcasecmp(sbtype,"sbpro1")) type=SBT_PRO1;
 		else if (!strcasecmp(sbtype,"sbpro2")) type=SBT_PRO2;
 		else if (!strcasecmp(sbtype,"sb16")) type=SBT_16;
+		else if (!strcasecmp(sbtype,"gb")) type=SBT_GB;
 		else if (!strcasecmp(sbtype,"none")) type=SBT_NONE;
 		else type=SBT_16;
 
@@ -1455,13 +1497,23 @@ private:
 		/* Else assume auto */
 		else {
 			switch (type) {
-			case SBT_NONE:opl_mode=OPL_none;break;
-			case SBT_1:opl_mode=OPL_opl2;break;
-			case SBT_2:opl_mode=OPL_opl2;break;
-			case SBT_PRO1:opl_mode=OPL_dualopl2;break;
+			case SBT_NONE:
+				opl_mode=OPL_none;
+				break;
+			case SBT_GB:
+				opl_mode=OPL_cms;
+				break;
+			case SBT_1:
+			case SBT_2:
+				opl_mode=OPL_opl2;
+				break;
+			case SBT_PRO1:
+				opl_mode=OPL_dualopl2;
+				break;
 			case SBT_PRO2:
 			case SBT_16:
-				opl_mode=OPL_opl3;break;
+				opl_mode=OPL_opl3;
+				break;
 			}
 		}	
 	}
@@ -1469,16 +1521,22 @@ public:
 	SBLASTER(Section* configuration):Module_base(configuration) {
 		Bitu i;
 		Section_prop * section=static_cast<Section_prop *>(configuration);
+
 		sb.hw.base=section->Get_hex("sbbase");
 		sb.hw.irq=section->Get_int("irq");
-		sb.hw.dma8=section->Get_int("dma");
-		sb.hw.dma16=section->Get_int("hdma");
+		Bitu dma8bit=section->Get_int("dma");
+		if (dma8bit>0xff) dma8bit=0xff;
+		sb.hw.dma8=(Bit8u)(dma8bit&0xff);
+		Bitu dma16bit=section->Get_int("hdma");
+		if (dma16bit>0xff) dma16bit=0xff;
+		sb.hw.dma16=(Bit8u)(dma16bit&0xff);
+
 		sb.mixer.enabled=section->Get_bool("sbmixer");
 		sb.mixer.stereo=false;
-		OPL_Mode opl_mode = OPL_none;
-		Find_Type_And_Opl(section,sb.type,opl_mode);
+
+		Find_Type_And_Opl(section,sb.type,oplmode);
 	
-		switch (opl_mode) {
+		switch (oplmode) {
 		case OPL_none:
 			WriteHandler[0].Install(0x388,adlib_gusforward,IO_MB);
 			break;
@@ -1488,14 +1546,17 @@ public:
 			break;
 		case OPL_opl2:
 			CMS_Init(section);
+			// fall-through
 		case OPL_dualopl2:
 		case OPL_opl3:
-			OPL_Init(section,opl_mode);
+			OPL_Init(section,oplmode);
 			break;
 		}
-		if (sb.type==SBT_NONE) return;
+		if (sb.type==SBT_NONE || sb.type==SBT_GB) return;
+
 		sb.chan=MixerChan.Install(&SBLASTER_CallBack,22050,"SB");
 		sb.dsp.state=DSP_S_NORMAL;
+		sb.dsp.out.lastval=0xaa;
 		sb.dma.chan=NULL;
 
 		for (i=4;i<=0xf;i++) {
@@ -1508,8 +1569,10 @@ public:
 		for (i=0;i<256;i++) ASP_regs[i] = 0;
 		ASP_regs[5] = 0x01;
 		ASP_regs[9] = 0xf8;
+
 		DSP_Reset();
 		CTMIXER_Reset();
+
 		// The documentation does not specify if SB gets initialized with the speaker enabled
 		// or disabled. Real SBPro2 has it disabled. 
 		sb.speaker=false;
@@ -1520,8 +1583,8 @@ public:
 		// Create set blaster line
 		ostringstream temp;
 		temp << "SET BLASTER=A" << setw(3)<< hex << sb.hw.base
-		     << " I" << dec << sb.hw.irq << " D"<< sb.hw.dma8;
-		if (sb.type==SBT_16) temp << " H" << sb.hw.dma16;
+		     << " I" << dec << (Bitu)sb.hw.irq << " D" << (Bitu)sb.hw.dma8;
+		if (sb.type==SBT_16) temp << " H" << (Bitu)sb.hw.dma16;
 		temp << " T" << static_cast<unsigned int>(sb.type) << ends;
 
 		autoexecline.Install(temp.str());
@@ -1532,34 +1595,28 @@ public:
 	}	
 	
 	~SBLASTER() {
-	Section_prop * section=static_cast<Section_prop *>(m_configuration);
-		OPL_Mode opl_mode = OPL_none;
-		Find_Type_And_Opl(section,sb.type,opl_mode);
-	
-		switch (opl_mode) {
+		switch (oplmode) {
 		case OPL_none:
-
 			break;
 		case OPL_cms:
-
-		CMS_ShutDown(m_configuration);
+			CMS_ShutDown(m_configuration);
 			break;
 		case OPL_opl2:
-		CMS_ShutDown(m_configuration);
+			CMS_ShutDown(m_configuration);
+			// fall-through
 		case OPL_dualopl2:
 		case OPL_opl3:
-		OPL_ShutDown(m_configuration);
+			OPL_ShutDown(m_configuration);
 			break;
 		}
-
-		if (sb.type==SBT_NONE) return;
-		DSP_Reset();//Stop everything	
+		if (sb.type==SBT_NONE || sb.type==SBT_GB) return;
+		DSP_Reset(); // Stop everything	
 	}	
 }; //End of SBLASTER class
 
 
 static SBLASTER* test;
-void SBLASTER_ShutDown(Section* sec) {
+void SBLASTER_ShutDown(Section* /*sec*/) {
 	delete test;	
 }
 
