@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: vga_tseng.cpp,v 1.5 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: vga_tseng.cpp,v 1.5 2009-05-27 09:15:41 qbix79 Exp $ */
 
 
 #include "dosbox.h"
@@ -357,9 +357,10 @@ void FinishSetMode_ET4K(Bitu crtc_base, VGA_ModeExtraData* modeData) {
 		Bitu best = 1;
 		Bits dist = 100000000;
 		for (Bitu i=0; i<16; i++) {
-			if (abs(target-et4k.clockFreq[i]) < dist) {
+			Bits cdiff=abs((Bits)(target-et4k.clockFreq[i]));
+			if (cdiff < dist) {
 				best = i;
-				dist = abs(target-et4k.clockFreq[i]);
+				dist = cdiff;
 			}
 		}
 		set_clock_index_et4k(best);
@@ -718,9 +719,10 @@ void FinishSetMode_ET3K(Bitu crtc_base, VGA_ModeExtraData* modeData) {
 		Bitu best = 1;
 		Bits dist = 100000000;
 		for (Bitu i=0; i<8; i++) {
-			if (abs(target-et3k.clockFreq[i]) < dist) {
+			Bits cdiff = abs((Bits)(target-et3k.clockFreq[i]));
+			if (cdiff < dist) {
 				best = i;
-				dist = abs(target-et3k.clockFreq[i]);
+				dist = cdiff;
 			}
 		}
 		set_clock_index_et3k(best);
